## Map Matching

The Mapbox Map Matching API lets you use fuzzy, innaccurate traces from a
GPS unit or a phone and snap them to the OpenStreetMap
road and path network, producing clean paths that can be displayed on a map
or used for other analysis.

| Restriction | |
| --- | --- |
| Requests per minute | 60 |
| Coordinates per request | 100 |
| Usage | Results must be displayed on a Mapbox map |

The Map Matching API is limited to 60 requests per minute and results must
be displayed on a Mapbox map using one of our [SDKs](https://mapbox.com/developers/).
For high volume or other use cases, [contact us](https://mapbox.com/contact/).

### Retrieve a map match

```endpoint
POST /matching/v4/{profile}.json
```

The `{profile}` parameter of your request should be a Directions profile ID. Supported IDs
are:

- `mapbox.driving` for car traces
- `mapbox.walking` for pedestrian and hiking traces
- `mapbox.cycling` for bicycle traces

#### Example

```curl
$ curl -X POST \
  --header "Content-Type:application/json" \
  -d @trace.json \
  https://api.mapbox.com/matching/v4/mapbox.driving.json
```

```python
>>> from mapbox import MapMatcher
>>> service = MapMatcher()
>>> response = service.match(line, profile='mapbox.driving')
>>> response.status_code
200
>>> response.headers['Content-Type']
'application/json; charset=utf-8'
```

```javascript
var mapboxClient = new MapboxClient('ACCESSTOKEN');

mapboxClient.matching({
  "type": "Feature",
  "properties": {
    "coordTimes": [
      "2015-04-21T06:00:10Z",
      "2015-04-21T06:00:15Z",
      "2015-04-21T06:00:20Z"
    ]
  },
  "geometry": {
    "type": "LineString",
    "coordinates": [
      [13.419756889343262, 52.50171780290061],
      [13.419885635375975, 52.50237416816131],
      [13.420631289482117, 52.50294888790448]
    ]
  }
}, function(err, res) {
  // res is a document with directions
});
```

**Parameters**

parameter | description
----------|----------------------|
`geometry`<b /> optional | Format of the returned geometry. Allowed values are: `geojson` (default, as [LineString](http://geojson.org/geojson-spec.html#linestring)), `polyline` ([documentation](https://developers.google.com/maps/documentation/utilities/polylinealgorithm)) with precision 6, `false` (no geometry, but `matched points`)
`gps_precision`<br /> optional | An integer in meters indicating the assumed precision of the used tracking device. Use higher numbers (5-10) for noisy traces and lower numbers (1-3) for clean traces. The default value is 4.

**Request body format**

Requests must set the `Content-Type` header to `application/json`.

Request body must be one [GeoJSON Feature](http://geojson.org/geojson-spec.html#feature-objects)
with a [LineString geometry](http://geojson.org/geojson-spec.html#linestring).
The LineString may have up to 100 positions.

Geometry coordinates may have corresponding timestamps. It is recommended to
include timestamps, since they help improve the quality of the matching.
The timestamps are located in the `coordTimes` array. The timestamps may either
be numbers in [Unix time](https://en.wikipedia.org/wiki/Unix_time) (seconds since the Unix epoch) or strings
in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format. The timestamps
must be ascending. For best results, assure that your timestamps have a sample
rate of about 5 seconds.

#### Example request body

```json
{
  "type": "Feature",
  "properties": {
    "coordTimes": [
      "2015-04-21T06:00:00Z",
      "2015-04-21T06:00:05Z",
      "2015-04-21T06:00:10Z",
      "2015-04-21T06:00:15Z",
      "2015-04-21T06:00:20Z"
    ]
    },
  "geometry": {
    "type": "LineString",
    "coordinates": [
      [13.418946862220764, 52.50055852688439],
      [13.419011235237122, 52.50113000479732],
      [13.419756889343262, 52.50171780290061],
      [13.419885635375975, 52.50237416816131],
      [13.420631289482117, 52.50294888790448]
    ]
  }
}
```


**Response format**

The response is returned as a [GeoJSON FeatureCollection](http://geojson.org/geojson-spec.html#feature-collection-objects)
which may include one or more [GeoJSON Features](http://geojson.org/geojson-spec.html#feature-objects).
On clean matches, only one feature is returned. When the matching algorithm cannot
determine the correct match between two points, it will omit that line and create
several sub-matches, each as a feature.

Depending on the requested format, each feature will include a `geometry` in [LineString](http://geojson.org/geojson-spec.html#linestring)
or [Polyline](https://developers.google.com/maps/documentation/utilities/polylinealgorithm) format.
The geometries will be aligned with the OpenStreetMap road/path network.

Each feature includes a `confidence` value which is a number between `0`
and `1`, with `0` being very low confidence and `1` being very high confidence.
This value is an indicator of how well the algorithm was able to match the trace.
The number of features in the result also indicates match quality: more
features indicate problems with trace's alignment that required splitting.

Each feature includes `matchedPoints`. These let you determine the matches between
your original trace points and the returned points. Since a response may include
several features, the `indices` array lets you connect each value in
`matchedPoints` to its original trace point, with the array index as the position
in the feature's `matchedPoints` array and the value as the position in the
original trace coordinates array.

#### Example response

```json
{
  "type": "FeatureCollection",
  "features": [
    {
      "type": "Feature",
      "geometry": {
        "type": "LineString",
        "coordinates": [
          [13.418805, 52.500599],
          [13.418851, 52.500659],
          [13.419145, 52.501095],
          [13.419276, 52.501286],
          [13.419618, 52.501754],
          [13.419981, 52.502249],
          [13.420042, 52.502335],
          [13.420494, 52.502984]
        ]
      },
      "properties": {
        "confidence": 0.8165256193474186,
        "matchedPoints": [
          [13.418805122375488, 52.50059890747071],
          [13.419144630432129, 52.50109481811523],
          [13.419617652893066, 52.50175094604492],
          [13.420042037963867, 52.50233459472656],
          [13.420494079589844, 52.50298309326172]
        ],
        "indices": [0, 1, 2, 3, 4]
      }
    }
  ]
}
```

**Preprocessing traces**

If your traces are in `GPX` or `KML` format, you can
use [toGeoJSON](https://github.com/mapbox/togeojson) to transform them to GeoJSON.

Some pre-processing tips to achieve the best results:

- The Map Matching API is limited to processing traces with up to 100 coordinates.
  If you need to process longer traces, you can split the trace and make
  multiple requests.
- Clusters of points (e.g. a person waiting at a train crossing for a few minutes)
  often don't add more information to a trace and can be removed.
- Map Matching works best with a sample rate of 5 seconds between points.
  If your trace has a higher sample rate, you may want to downsample your trace.
- [geojson-tidy](https://github.com/mapbox/geojson-tidy) can be useful for preprocessing:
  it removes clusters and re-samples traces.
